@@ -16,7 +16,6 @@
 
 #include <boost/filesystem/path.hpp>
 #include <boost/filesystem/operations.hpp>
-#include <boost/nowide/cenv.hpp>
 #include <boost/nowide/fstream.hpp>
 #include <boost/property_tree/ini_parser.hpp>
 #include <boost/property_tree/ptree_fwd.hpp>
@@ -102,9 +101,6 @@ void AppConfig::set_defaults()
         if (get("associate_stl").empty())
             set("associate_stl", "0");
 
-        if (get("tabs_as_menu").empty())
-            set("tabs_as_menu", "0");
-
         if (get("suppress_round_corners").empty())
             set("suppress_round_corners", "1");
 #endif // _WIN32
@@ -113,11 +109,6 @@ void AppConfig::set_defaults()
         if (!get("use_legacy_opengl").empty())
             erase("", "use_legacy_opengl");
 
-#ifdef __APPLE__
-        if (get("use_retina_opengl").empty())
-            set("use_retina_opengl", "1");
-#endif
-
         if (get("single_instance").empty())
             set("single_instance", 
 #ifdef __APPLE__
@@ -186,6 +177,11 @@ void AppConfig::set_defaults()
 #endif // _WIN32
     }
 
+#ifdef __APPLE__
+    if (get("use_retina_opengl").empty())
+        set("use_retina_opengl", "1");
+#endif // __APPLE__
+
     if (get("seq_top_layer_only").empty())
         set("seq_top_layer_only", "1");
 
@@ -216,6 +212,21 @@ void AppConfig::set_defaults()
     if (get("wifi_config_dialog_declined").empty())
         set("wifi_config_dialog_declined", "0");
 
+    if (get("connect_polling").empty())
+        set("connect_polling", "1");
+
+    if (get("auth_login_dialog_confirmed").empty())
+        set("auth_login_dialog_confirmed", "0");
+
+    if (get("show_estimated_times_in_dbl_slider").empty())
+        set("show_estimated_times_in_dbl_slider", "1");
+
+    if (get("show_ruler_in_dbl_slider").empty())
+        set("show_ruler_in_dbl_slider", "0");
+
+    if (get("show_ruler_bg_in_dbl_slider").empty())
+        set("show_ruler_bg_in_dbl_slider", "1");
+
 #ifdef _WIN32
     if (get("use_legacy_3DConnexion").empty())
         set("use_legacy_3DConnexion", "0");
