@@ -56,14 +56,19 @@ void FillConcentric::_fill_surface_single(
     // clip the paths to prevent the extruder from getting exactly on the first point of the loop
     // Keep valid paths only.
     size_t j = iPathFirst;
-    for (size_t i = iPathFirst; i < polylines_out.size(); ++ i) {
+    for (size_t i = iPathFirst; i < polylines_out.size(); ++i) {
         polylines_out[i].clip_end(this->loop_clipping);
         if (polylines_out[i].is_valid()) {
+            if (params.prefer_clockwise_movements)
+                polylines_out[i].reverse();
+
             if (j < i)
                 polylines_out[j] = std::move(polylines_out[i]);
-            ++ j;
+
+            ++j;
         }
     }
+
     if (j < polylines_out.size())
         polylines_out.erase(polylines_out.begin() + int(j), polylines_out.end());
     //TODO: return ExtrusionLoop objects to get better chained paths,
@@ -94,6 +99,7 @@ void FillConcentric::_fill_surface_single(const FillParams              &params,
         for (Arachne::VariableWidthLines &loop : loops) {
             if (loop.empty())
                 continue;
+
             for (const Arachne::ExtrusionLine &wall : loop)
                 all_extrusions.emplace_back(&wall);
         }
@@ -106,8 +112,14 @@ void FillConcentric::_fill_surface_single(const FillParams              &params,
                 continue;
 
             ThickPolyline thick_polyline = Arachne::to_thick_polyline(*extrusion);
-            if (extrusion->is_closed)
+            if (extrusion->is_closed) {
+                // Arachne produces contour with clockwise orientation and holes with counterclockwise orientation.
+                if (const bool extrusion_reverse = params.prefer_clockwise_movements ? !extrusion->is_contour() : extrusion->is_contour(); extrusion_reverse)
+                    thick_polyline.reverse();
+
                 thick_polyline.start_at_index(nearest_point_index(thick_polyline.points, last_pos));
+            }
+
             thick_polylines_out.emplace_back(std::move(thick_polyline));
             last_pos = thick_polylines_out.back().last_point();
         }
