@@ -21,11 +21,10 @@ class AvoidCrossingPerimeters
 public:
     // Routing around the objects vs. inside a single object.
     void        use_external_mp(bool use = true) { m_use_external_mp = use; };
-    void        use_external_mp_once()  { m_use_external_mp_once = true; }
-    bool        used_external_mp_once() { return m_use_external_mp_once; }
+    bool        used_external_mp_once() { return use_external_mp_once; }
     void        disable_once()          { m_disabled_once = true; }
     bool        disabled_once() const   { return m_disabled_once; }
-    void        reset_once_modifiers()  { m_use_external_mp_once = false; m_disabled_once = false; }
+    void        reset_once_modifiers()  { use_external_mp_once = false; m_disabled_once = false; }
 
     void        init_layer(const Layer &layer);
 
@@ -54,10 +53,10 @@ public:
         }
     };
 
+    // just for the next travel move
+    bool           use_external_mp_once { false };
 private:
     bool           m_use_external_mp { false };
-    // just for the next travel move
-    bool           m_use_external_mp_once { false };
     // this flag disables avoid_crossing_perimeters just for the next travel move
     // we enable it by default for the first travel move in print
     bool           m_disabled_once { true };
