@@ -131,6 +131,10 @@ bool GCodeReader::parse_file_raw_internal(const std::string &filename, ParseLine
 {
     FilePtr in{ boost::nowide::fopen(filename.c_str(), "rb") };
 
+    fseek(in.f, 0, SEEK_END);
+    const long file_size = ftell(in.f);
+    rewind(in.f);
+
     // Read the input stream 64kB at a time, extract lines and process them.
     std::vector<char> buffer(65536 * 10, 0);
     // Line buffer.
@@ -141,6 +145,7 @@ bool GCodeReader::parse_file_raw_internal(const std::string &filename, ParseLine
         size_t cnt_read = ::fread(buffer.data(), 1, buffer.size(), in.f);
         if (::ferror(in.f))
             return false;
+
         bool eof       = cnt_read == 0;
         auto it        = buffer.begin();
         auto it_bufend = buffer.begin() + cnt_read;
@@ -178,6 +183,8 @@ bool GCodeReader::parse_file_raw_internal(const std::string &filename, ParseLine
         if (eof)
             break;
         file_pos += cnt_read;
+        if (m_progress_callback != nullptr)
+            m_progress_callback(static_cast<float>(file_pos) / static_cast<float>(file_size));
     }
     return true;
 }
@@ -199,10 +206,11 @@ bool GCodeReader::parse_file(const std::string &file, callback_t callback)
     return this->parse_file_internal(file, callback, [](size_t){});
 }
 
-bool GCodeReader::parse_file(const std::string &file, callback_t callback, std::vector<size_t> &lines_ends)
+bool GCodeReader::parse_file(const std::string& file, callback_t callback, std::vector<std::vector<size_t>>& lines_ends)
 {
     lines_ends.clear();
-    return this->parse_file_internal(file, callback, [&lines_ends](size_t file_pos){ lines_ends.emplace_back(file_pos); });
+    lines_ends.push_back(std::vector<size_t>());
+    return this->parse_file_internal(file, callback, [&lines_ends](size_t file_pos) { lines_ends.front().emplace_back(file_pos); });
 }
 
 bool GCodeReader::parse_file_raw(const std::string &filename, raw_line_callback_t line_callback)
@@ -236,42 +244,56 @@ const char* GCodeReader::axis_pos(const char *raw_str, char axis)
 
 bool GCodeReader::GCodeLine::has(char axis) const
 {
-    const char *c = axis_pos(m_raw.c_str(), axis);
-    return c != nullptr;
+    return GCodeReader::axis_pos(this->raw().c_str(), axis);
+}
+
+std::string_view GCodeReader::GCodeLine::axis_pos(char axis) const
+{ 
+    const std::string &s = this->raw();
+    const char *c = GCodeReader::axis_pos(this->raw().c_str(), axis);
+    return c ? std::string_view{ c, s.size() - (c - s.data()) } : std::string_view();
+}
+
+bool GCodeReader::GCodeLine::has_value(std::string_view axis_pos, float &value)
+{
+    if (const char *c = axis_pos.data(); c) {
+        // Try to parse the numeric value.
+        double v = 0.;
+        const char *end = axis_pos.data() + axis_pos.size();
+        auto [pend, ec] = fast_float::from_chars(++ c, end, v);
+        if (pend != c && is_end_of_word(*pend)) {
+            // The axis value has been parsed correctly.
+            value = float(v);
+            return true;
+        }
+    }
+    return false;
 }
 
 bool GCodeReader::GCodeLine::has_value(char axis, float &value) const
 {
     assert(is_decimal_separator_point());
-    const char *c = axis_pos(m_raw.c_str(), axis);
-    if (c == nullptr)
-        return false;
-    // Try to parse the numeric value.
-    double v = 0.;
-    const char* end = m_raw.c_str() + m_raw.size();
-    auto [pend, ec] = fast_float::from_chars(++c, end, v);
-    if (pend != c && is_end_of_word(*pend)) {
-        // The axis value has been parsed correctly.
-        value = float(v);
-        return true;
+    return this->has_value(this->axis_pos(axis), value);
+}
+
+bool GCodeReader::GCodeLine::has_value(std::string_view axis_pos, int &value)
+{
+    if (const char *c = axis_pos.data(); c) {
+        // Try to parse the numeric value.
+        char   *pend = nullptr;
+        long    v = strtol(++ c, &pend, 10);
+        if (pend != nullptr && is_end_of_word(*pend)) {
+            // The axis value has been parsed correctly.
+            value = int(v);
+            return true;
+        }
     }
     return false;
 }
 
 bool GCodeReader::GCodeLine::has_value(char axis, int &value) const
 {
-    const char *c = axis_pos(m_raw.c_str(), axis);
-    if (c == nullptr)
-        return false;
-    // Try to parse the numeric value.
-    char   *pend = nullptr;
-    long    v = strtol(++ c, &pend, 10);
-    if (pend != nullptr && is_end_of_word(*pend)) {
-        // The axis value has been parsed correctly.
-        value = int(v);
-        return true;
-    }
-    return false;
+    return this->has_value(this->axis_pos(axis), value);
 }
 
 void GCodeReader::GCodeLine::set(const GCodeReader &reader, const Axis axis, const float new_value, const int decimal_digits)
