@@ -35,6 +35,8 @@ VoronoiDiagram::construct_voronoi(const SegmentIterator segment_begin, const Seg
                 BOOST_LOG_TRIVIAL(warning) << "Detected Voronoi edge intersecting input segment, input polygons will be rotated back and forth.";
             } else if (m_issue_type == IssueType::FINITE_EDGE_WITH_NON_FINITE_VERTEX) {
                 BOOST_LOG_TRIVIAL(warning) << "Detected finite Voronoi vertex with non finite vertex, input polygons will be rotated back and forth.";
+            } else if (m_issue_type == IssueType::PARABOLIC_VORONOI_EDGE_WITHOUT_FOCUS_POINT) {
+                BOOST_LOG_TRIVIAL(warning) << "Detected parabolic Voronoi edges without focus point, input polygons will be rotated back and forth.";
             } else {
                 BOOST_LOG_TRIVIAL(error) << "Detected unknown Voronoi diagram issue, input polygons will be rotated back and forth.";
             }
@@ -48,6 +50,8 @@ VoronoiDiagram::construct_voronoi(const SegmentIterator segment_begin, const Seg
                     BOOST_LOG_TRIVIAL(error) << "Detected Voronoi edge intersecting input segment even after the rotation of input.";
                 } else if (m_issue_type == IssueType::FINITE_EDGE_WITH_NON_FINITE_VERTEX) {
                     BOOST_LOG_TRIVIAL(error) << "Detected finite Voronoi vertex with non finite vertex even after the rotation of input.";
+                } else if (m_issue_type == IssueType::PARABOLIC_VORONOI_EDGE_WITHOUT_FOCUS_POINT) {
+                    BOOST_LOG_TRIVIAL(error) << "Detected parabolic Voronoi edges without focus point even after the rotation of input.";
                 } else {
                     BOOST_LOG_TRIVIAL(error) << "Detected unknown Voronoi diagram issue even after the rotation of input.";
                 }
@@ -147,6 +151,9 @@ void VoronoiDiagram::copy_to_local(voronoi_diagram_type &voronoi_diagram) {
             new_edge.prev(&m_edges[prev_edge_idx]);
         }
     }
+
+    m_voronoi_diagram.clear();
+    m_is_modified = true;
 }
 
 template<typename SegmentIterator>
@@ -156,8 +163,8 @@ typename boost::polygon::enable_if<
     VoronoiDiagram::IssueType>::type
 VoronoiDiagram::detect_known_issues(const VoronoiDiagram &voronoi_diagram, SegmentIterator segment_begin, SegmentIterator segment_end)
 {
-    if (has_finite_edge_with_non_finite_vertex(voronoi_diagram)) {
-        return IssueType::FINITE_EDGE_WITH_NON_FINITE_VERTEX;
+    if (const IssueType edge_issue_type = detect_known_voronoi_edge_issues(voronoi_diagram); edge_issue_type != IssueType::NO_ISSUE_DETECTED) {
+        return edge_issue_type;
     } else if (const IssueType cell_issue_type = detect_known_voronoi_cell_issues(voronoi_diagram, segment_begin, segment_end); cell_issue_type != IssueType::NO_ISSUE_DETECTED) {
         return cell_issue_type;
     } else if (!VoronoiUtilsCgal::is_voronoi_diagram_planar_angle(voronoi_diagram, segment_begin, segment_end)) {
@@ -215,16 +222,20 @@ VoronoiDiagram::detect_known_voronoi_cell_issues(const VoronoiDiagram &voronoi_d
     return IssueType::NO_ISSUE_DETECTED;
 }
 
-bool VoronoiDiagram::has_finite_edge_with_non_finite_vertex(const VoronoiDiagram &voronoi_diagram)
+VoronoiDiagram::IssueType VoronoiDiagram::detect_known_voronoi_edge_issues(const VoronoiDiagram &voronoi_diagram)
 {
     for (const voronoi_diagram_type::edge_type &edge : voronoi_diagram.edges()) {
         if (edge.is_finite()) {
             assert(edge.vertex0() != nullptr && edge.vertex1() != nullptr);
             if (edge.vertex0() == nullptr || edge.vertex1() == nullptr || !VoronoiUtils::is_finite(*edge.vertex0()) || !VoronoiUtils::is_finite(*edge.vertex1()))
-                return true;
+                return IssueType::FINITE_EDGE_WITH_NON_FINITE_VERTEX;
+
+            if (edge.is_curved() && !edge.cell()->contains_point() && !edge.twin()->cell()->contains_point())
+                return IssueType::PARABOLIC_VORONOI_EDGE_WITHOUT_FOCUS_POINT;
         }
     }
-    return false;
+
+    return IssueType::NO_ISSUE_DETECTED;
 }
 
 template<typename SegmentIterator>
@@ -345,9 +356,6 @@ VoronoiDiagram::try_to_repair_degenerated_voronoi_diagram_by_rotation(const Segm
     for (vertex_type &vertex : m_vertices)
         vertex.color(0);
 
-    m_voronoi_diagram.clear();
-    m_is_modified = true;
-
     return issue_type;
 }
 
