@@ -27,6 +27,7 @@
 #include "enum_bitmask.hpp"
 #include "TextConfiguration.hpp"
 #include "EmbossShape.hpp"
+#include "TriangleSelector.hpp"
 
 #include <map>
 #include <memory>
@@ -56,7 +57,6 @@ class ModelVolume;
 class ModelWipeTower;
 class Print;
 class SLAPrint;
-class TriangleSelector;
 
 namespace UndoRedo {
 	class StackImpl;
@@ -654,29 +654,6 @@ private:
     void update_min_max_z();
 };
 
-enum class EnforcerBlockerType : int8_t {
-    // Maximum is 3. The value is serialized in TriangleSelector into 2 bits.
-    NONE      = 0,
-    ENFORCER  = 1,
-    BLOCKER   = 2,
-    // Maximum is 15. The value is serialized in TriangleSelector into 6 bits using a 2 bit prefix code.
-    Extruder1 = ENFORCER,
-    Extruder2 = BLOCKER,
-    Extruder3,
-    Extruder4,
-    Extruder5,
-    Extruder6,
-    Extruder7,
-    Extruder8,
-    Extruder9,
-    Extruder10,
-    Extruder11,
-    Extruder12,
-    Extruder13,
-    Extruder14,
-    Extruder15,
-};
-
 enum class ConversionType : int {
     CONV_TO_INCH,
     CONV_FROM_INCH,
@@ -687,14 +664,14 @@ enum class ConversionType : int {
 class FacetsAnnotation final : public ObjectWithTimestamp {
 public:
     // Assign the content if the timestamp differs, don't assign an ObjectID.
-    void assign(const FacetsAnnotation& rhs) { if (! this->timestamp_matches(rhs)) { m_data = rhs.m_data; this->copy_timestamp(rhs); } }
-    void assign(FacetsAnnotation&& rhs) { if (! this->timestamp_matches(rhs)) { m_data = std::move(rhs.m_data); this->copy_timestamp(rhs); } }
-    const std::pair<std::vector<std::pair<int, int>>, std::vector<bool>>& get_data() const throw() { return m_data; }
+    void assign(const FacetsAnnotation &rhs) { if (! this->timestamp_matches(rhs)) { m_data = rhs.m_data; this->copy_timestamp(rhs); } }
+    void assign(FacetsAnnotation &&rhs) { if (! this->timestamp_matches(rhs)) { m_data = std::move(rhs.m_data); this->copy_timestamp(rhs); } }
+    const TriangleSelector::TriangleSplittingData &get_data() const noexcept { return m_data; }
     bool set(const TriangleSelector& selector);
-    indexed_triangle_set get_facets(const ModelVolume& mv, EnforcerBlockerType type) const;
-    indexed_triangle_set get_facets_strict(const ModelVolume& mv, EnforcerBlockerType type) const;
-    bool has_facets(const ModelVolume& mv, EnforcerBlockerType type) const;
-    bool empty() const { return m_data.first.empty(); }
+    indexed_triangle_set get_facets(const ModelVolume& mv, TriangleStateType type) const;
+    indexed_triangle_set get_facets_strict(const ModelVolume& mv, TriangleStateType type) const;
+    bool has_facets(const ModelVolume& mv, TriangleStateType type) const;
+    bool empty() const { return m_data.triangles_to_split.empty(); }
 
     // Following method clears the config and increases its timestamp, so the deleted
     // state is considered changed from perspective of the undo/redo stack.
@@ -704,11 +681,11 @@ public:
     std::string get_triangle_as_string(int i) const;
 
     // Before deserialization, reserve space for n_triangles.
-    void reserve(int n_triangles) { m_data.first.reserve(n_triangles); }
+    void reserve(int n_triangles) { m_data.triangles_to_split.reserve(n_triangles); }
     // Deserialize triangles one by one, with strictly increasing triangle_id.
     void set_triangle_from_string(int triangle_id, const std::string& str);
     // After deserializing the last triangle, shrink data to fit.
-    void shrink_to_fit() { m_data.first.shrink_to_fit(); m_data.second.shrink_to_fit(); }
+    void shrink_to_fit() { m_data.triangles_to_split.shrink_to_fit(); m_data.bitstream.shrink_to_fit(); }
 
 private:
     // Constructors to be only called by derived classes.
@@ -718,9 +695,9 @@ private:
     // by an existing ID copied from elsewhere.
     explicit FacetsAnnotation(int) : ObjectWithTimestamp(-1) {}
     // Copy constructor copies the ID.
-    explicit FacetsAnnotation(const FacetsAnnotation &rhs) = default;
+    FacetsAnnotation(const FacetsAnnotation &rhs) = default;
     // Move constructor copies the ID.
-    explicit FacetsAnnotation(FacetsAnnotation &&rhs) = default;
+    FacetsAnnotation(FacetsAnnotation &&rhs) = default;
 
     // called by ModelVolume::assign_copy()
     FacetsAnnotation& operator=(const FacetsAnnotation &rhs) = default;
@@ -729,12 +706,9 @@ private:
     friend class cereal::access;
     friend class UndoRedo::StackImpl;
 
-    template<class Archive> void serialize(Archive &ar)
-    {
-        ar(cereal::base_class<ObjectWithTimestamp>(this), m_data);
-    }
+    template<class Archive> void serialize(Archive &ar) { ar(cereal::base_class<ObjectWithTimestamp>(this), m_data); }
 
-    std::pair<std::vector<std::pair<int, int>>, std::vector<bool>> m_data;
+    TriangleSelector::TriangleSplittingData m_data;
 
     // To access set_new_unique_id() when copy / pasting a ModelVolume.
     friend class ModelVolume;
@@ -936,6 +910,14 @@ public:
     bool is_seam_painted() const { return !this->seam_facets.empty(); }
     bool is_mm_painted() const { return !this->mm_segmentation_facets.empty(); }
 
+    // Returns 0-based indices of extruders painted by multi-material painting gizmo.
+    std::vector<size_t> get_extruders_from_multi_material_painting() const;
+
+    static size_t get_extruder_color_idx(const ModelVolume& model_volume, const int extruders_count) {
+        const int extruder_id = model_volume.extruder_id();
+        return (extruder_id <= 0 || extruder_id > extruders_count) ? 0 : extruder_id - 1;
+    }
+
 protected:
 	friend class Print;
     friend class SLAPrint;
