@@ -39,6 +39,7 @@ public:
     Semver                          config_version;
     std::string                     config_update_url;
     std::string                     changelog_url;
+    std::string                     repo_id;
     bool                            templates_profile { false };
 
     struct PrinterVariant {
@@ -125,6 +126,7 @@ public:
         TYPE_PHYSICAL_PRINTER,
         // This type is here to support search through the Preferences
         TYPE_PREFERENCES,
+        TYPE_WEBVIEW,
     };
 
     Type                type        = TYPE_INVALID;
@@ -409,6 +411,7 @@ public:
     PresetWithVendorProfile get_edited_preset_with_vendor_profile() const { return this->get_preset_with_vendor_profile(this->get_edited_preset()); }
 
     const std::string& 		get_preset_name_by_alias(const std::string& alias) const;
+    const std::string&      get_preset_name_by_alias_invisible(const std::string& alias) const;
 	const std::string*		get_preset_name_renamed(const std::string &old_name) const;
 
 	// used to update preset_choice from Tab
@@ -436,6 +439,8 @@ public:
     const Preset*   find_preset(const std::string &name, bool first_visible_if_not_found = false, bool respect_active_preset = true) const 
         { return const_cast<PresetCollection*>(this)->find_preset(name, first_visible_if_not_found, respect_active_preset); }
 
+    size_t          get_preset_idx_by_name(const std::string preset_name) const;
+
     size_t          first_visible_idx() const;
     // Return index of the first compatible preset. Certainly at least the '- default -' preset shall be compatible.
     // If one of the prefered_alternates is compatible, select it.
@@ -525,7 +530,8 @@ public:
     // Select a profile by its name. Return true if the selection changed.
     // Without force, the selection is only updated if the index changes.
     // With force, the changes are reverted if the new index is the same as the old index.
-    bool            select_preset_by_name(const std::string &name, bool force);
+    // With force_invisible, force preset selection even it's invisible.
+    bool            select_preset_by_name(const std::string &name, bool force, bool force_invisible = false);
 
     // Generate a file path from a profile name. Add the ".ini" suffix if it is missing.
     std::string     path_from_name(const std::string &new_name) const;
