@@ -219,10 +219,8 @@ static t_config_option_keys print_config_diffs(const StaticPrintConfig     &curr
         "branchingsupport_head_width"sv,
         "branchingsupport_pillar_diameter"sv,
         "support_points_density_relative"sv,
-        "relative_correction_x"sv,
-        "relative_correction_y"sv,
-        "relative_correction_z"sv,
         "elefant_foot_compensation"sv,
+        "absolute_correction"sv,
     };
 
     static constexpr auto material_ow_prefix = "material_ow_";
@@ -685,6 +683,12 @@ std::string SLAPrint::validate(std::vector<std::string>*) const
         }
     }
 
+    if ((!m_material_config.use_tilt.get_at(0) && m_material_config.tower_hop_height.get_at(0) == 0)
+        || (!m_material_config.use_tilt.get_at(1) && m_material_config.tower_hop_height.get_at(1) == 0))
+        return _u8L("Disabling the 'Use tilt' function causes the object to separate away from the film in the "
+                    "vertical direction only. Therefore, it is necessary to set the 'Tower hop height' parameter "
+                    "to reasonable value. The recommended value is 5 mm.");
+
     return "";
 }
 
@@ -838,6 +842,7 @@ bool SLAPrint::invalidate_state_by_config_options(const std::vector<t_config_opt
         "absolute_correction"sv,
         "elefant_foot_compensation"sv,
         "elefant_foot_min_width"sv,
+        "zcorrection_layers"sv,
         "gamma_correction"sv,
     };
 
@@ -858,7 +863,26 @@ bool SLAPrint::invalidate_state_by_config_options(const std::vector<t_config_opt
         "display_mirror_y"sv,
         "display_orientation"sv,
         "sla_archive_format"sv,
-        "sla_output_precision"sv
+        "sla_output_precision"sv,
+        // tilt params
+        "delay_before_exposure"sv,
+        "delay_after_exposure"sv,
+        "tower_hop_height"sv,
+        "tower_speed"sv,
+        "use_tilt"sv,
+        "tilt_down_initial_speed"sv,
+        "tilt_down_offset_steps"sv,
+        "tilt_down_offset_delay"sv,
+        "tilt_down_finish_speed"sv,
+        "tilt_down_cycles"sv,
+        "tilt_down_delay"sv,
+        "tilt_up_initial_speed"sv,
+        "tilt_up_offset_steps"sv,
+        "tilt_up_offset_delay"sv,
+        "tilt_up_finish_speed"sv,
+        "tilt_up_cycles"sv,
+        "tilt_up_delay"sv,
+        "area_fill"sv,
     };
 
     static StaticSet steps_ignore = {
@@ -869,7 +893,6 @@ bool SLAPrint::invalidate_state_by_config_options(const std::vector<t_config_opt
         "fast_tilt_time"sv,
         "slow_tilt_time"sv,
         "high_viscosity_tilt_time"sv,
-        "area_fill"sv,
         "bottle_cost"sv,
         "bottle_volume"sv,
         "bottle_weight"sv,
@@ -884,9 +907,8 @@ bool SLAPrint::invalidate_state_by_config_options(const std::vector<t_config_opt
         "material_ow_branchingsupport_head_width"sv,
         "material_ow_elefant_foot_compensation"sv,
         "material_ow_support_points_density_relative"sv,
-        "material_ow_relative_correction_x"sv,
-        "material_ow_relative_correction_y"sv,
-        "material_ow_relative_correction_z"sv
+        "material_ow_absolute_correction"sv,
+        "printer_model"sv,
     };
 
     std::vector<SLAPrintStep> steps;
@@ -1200,7 +1222,7 @@ SLAPrintObject::get_parts_to_slice(SLAPrintObjectStep untilstep) const
 
     std::vector<csg::CSGPart> ret;
 
-    for (unsigned int step = 0; step < s; ++step) {
+    for (unsigned int step = 0; step <= s; ++step) {
         auto r = m_mesh_to_slice.equal_range(SLAPrintObjectStep(step));
         csg::copy_csgrange_shallow(Range{r.first, r.second}, std::back_inserter(ret));
     }
