@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2018 - 2022 Lukáš Matěna @lukasmatena, Vojtěch Bubník @bubnikv, David Kocík @kocikdav, Vojtěch Král @vojtechkral
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include "Version.hpp"
 
 #include <cctype>
@@ -69,80 +73,7 @@ bool Version::is_current_slic3r_downgrade() const
 {
 	return Slic3r::SEMVER < min_slic3r_version;
 }
-#if 0
-//TODO: This test should be moved to a unit test, once we have C++ unit tests in place.
-static int version_test()
-{
-	Version v;
-	v.config_version 	 = *Semver::parse("1.1.2");
-	v.min_slic3r_version = *Semver::parse("1.38.0");
-	v.max_slic3r_version = Semver::inf();
-	assert(v.is_slic3r_supported(*Semver::parse("1.38.0")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.38.0-alpha")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.37.0-alpha")));
-	// Test the prerelease status.
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-rc2")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0")));
-	v.config_version 	 = *Semver::parse("1.1.2-alpha");
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha1")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0-beta")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0-rc2")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0")));
-	v.config_version 	 = *Semver::parse("1.1.2-alpha1");
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha1")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0-beta")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0-rc2")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0")));
-	v.config_version 	 = *Semver::parse("1.1.2-beta");
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0-rc")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0-rc2")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0")));
-	v.config_version 	 = *Semver::parse("1.1.2-rc");
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-rc")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-rc2")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0")));
-	v.config_version 	 = *Semver::parse("1.1.2-rc2");
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-alpha1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-beta1")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-rc")));
-	assert(v.is_slic3r_supported(*Semver::parse("1.39.0-rc2")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.39.0")));
-	// Test the upper boundary.
-	v.config_version 	 = *Semver::parse("1.1.2");
-	v.max_slic3r_version = *Semver::parse("1.39.3-beta1");
-	assert(v.is_slic3r_supported(*Semver::parse("1.38.0")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.38.0-alpha")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.38.0-alpha1")));
-	assert(! v.is_slic3r_supported(*Semver::parse("1.37.0-alpha")));
-	return 0;
-}
-static int version_test_run = version_test();
-#endif
+
 
 inline char* left_trim(char *c)
 {
@@ -307,9 +238,11 @@ Index::const_iterator Index::recommended() const
 std::vector<Index> Index::load_db()
 {
     boost::filesystem::path cache_dir = boost::filesystem::path(Slic3r::data_dir()) / "cache";
+    boost::filesystem::path vendor_dir = boost::filesystem::path(Slic3r::data_dir()) / "vendor";
 
     std::vector<Index> index_db;
     std::string errors_cummulative;
+    
 	for (auto &dir_entry : boost::filesystem::directory_iterator(cache_dir))
         if (Slic3r::is_idx_file(dir_entry)) {
         	Index idx;
@@ -323,6 +256,20 @@ std::vector<Index> Index::load_db()
             index_db.emplace_back(std::move(idx));
         }
 
+    for (auto &dir_entry : boost::filesystem::directory_iterator(vendor_dir))
+        if (Slic3r::is_idx_file(dir_entry)) {
+        	Index idx;
+            try {
+            	idx.load(dir_entry.path());
+            } catch (const std::runtime_error &err) {
+                errors_cummulative += err.what();
+                errors_cummulative += "\n";
+                continue;
+			}
+            if (std::find_if(index_db.begin(), index_db.end(), [idx](const Index& index) { return idx.vendor() == index.vendor();}) == index_db.end())
+                index_db.emplace_back(std::move(idx));
+        }
+
     if (! errors_cummulative.empty())
         throw Slic3r::RuntimeError(errors_cummulative);
     return index_db;
