@@ -16,6 +16,7 @@
 #include "libslic3r/Config.hpp"
 
 #include <boost/nowide/fstream.hpp>
+#include <boost/nowide/convert.hpp>
 #include <boost/filesystem.hpp>
 #include <boost/log/trivial.hpp>
 #include <boost/dll/runtime_symbol_info.hpp>
@@ -123,7 +124,7 @@ void resolve_path_from_var(const std::string& var, std::vector<std::string>& pat
     wxString wxdirs;
     if (! wxGetEnv(boost::nowide::widen(var), &wxdirs) || wxdirs.empty() )
         return;
-    std::string dirs = boost::nowide::narrow(wxdirs);
+    std::string dirs = into_u8(wxdirs);
     for (size_t i = dirs.find(':'); i != std::string::npos; i = dirs.find(':'))
     {
         paths.push_back(dirs.substr(0, i));
@@ -308,7 +309,7 @@ void DesktopIntegrationDialog::perform_desktop_integration()
         // if all failed - try creating default home folder
         if (i == target_candidates.size() - 1) {
             // create $HOME/.local/share
-              create_path(boost::nowide::narrow(wxFileName::GetHomeDir()), ".local/share/icons" + icon_theme_dirs);
+              create_path(into_u8(wxFileName::GetHomeDir()), ".local/share/icons" + icon_theme_dirs);
               // copy icon
              target_dir_icons = GUI::format("%1%/.local/share",wxFileName::GetHomeDir());
               std::string icon_path = GUI::format("%1%/icons/PrusaSlicer.png",resources_dir());
@@ -364,7 +365,7 @@ void DesktopIntegrationDialog::perform_desktop_integration()
     // if all failed - try creating default home folder
     if (!candidate_found) {
         // create $HOME/.local/share
-        create_path(boost::nowide::narrow(wxFileName::GetHomeDir()), ".local/share/applications");
+        create_path(into_u8(wxFileName::GetHomeDir()), ".local/share/applications");
         // create desktop file
         target_dir_desktop = GUI::format("%1%/.local/share", wxFileName::GetHomeDir());
         std::string path = GUI::format("%1%/applications/PrusaSlicer%2%.desktop", target_dir_desktop, version_suffix);
@@ -582,7 +583,7 @@ void DesktopIntegrationDialog::perform_downloader_desktop_integration()
     // if all failed - try creating default home folder
     if (!candidate_found) {
         // create $HOME/.local/share
-        create_path(boost::nowide::narrow(wxFileName::GetHomeDir()), ".local/share/applications");
+        create_path(into_u8(wxFileName::GetHomeDir()), ".local/share/applications");
         // create desktop file
         target_dir_desktop = GUI::format("%1%/.local/share", wxFileName::GetHomeDir());
         std::string path = GUI::format("%1%/applications/PrusaSlicerURLProtocol%2%.desktop", target_dir_desktop, version_suffix);
