@@ -29,6 +29,8 @@ class Print;
 
 namespace GUI {
 
+static constexpr char illegal_characters[] = "<>:/\\|?*\"";
+
 void disable_screensaver();
 void enable_screensaver();
 bool debugged();
@@ -42,9 +44,6 @@ extern AppConfig* get_app_config();
 
 extern void add_menus(wxMenuBar *menu, int event_preferences_changed, int event_language_change);
 
-// Change option value in config
-void change_opt_value(DynamicPrintConfig& config, const t_config_option_key& opt_key, const boost::any& value, int opt_index = 0);
-
 // If monospaced_font is true, the error message is displayed using html <code><pre></pre></code> tags,
 // so that the code formatting will be preserved. This is useful for reporting errors from the placeholder parser.
 void show_error(wxWindow* parent, const wxString& message, bool monospaced_font = false);
@@ -57,19 +56,6 @@ void warning_catcher(wxWindow* parent, const wxString& message);
 void show_substitutions_info(const PresetsConfigSubstitutions& presets_config_substitutions);
 void show_substitutions_info(const ConfigSubstitutions& config_substitutions, const std::string& filename);
 
-// Creates a wxCheckListBoxComboPopup inside the given wxComboCtrl, filled with the given text and items.
-// Items data must be separated by '|', and contain the item name to be shown followed by its initial value (0 for false, 1 for true).
-// For example "Item1|0|Item2|1|Item3|0", and so on.
-void create_combochecklist(wxComboCtrl* comboCtrl, const std::string& text, const std::string& items);
-
-// Returns the current state of the items listed in the wxCheckListBoxComboPopup contained in the given wxComboCtrl,
-// encoded inside an unsigned int.
-unsigned int combochecklist_get_flags(wxComboCtrl* comboCtrl);
-
-// Sets the current state of the items listed in the wxCheckListBoxComboPopup contained in the given wxComboCtrl,
-// with the flags encoded in the given unsigned int.
-void combochecklist_set_flags(wxComboCtrl* comboCtrl, unsigned int flags);
-
 // wxString conversions:
 
 // wxString from std::string in UTF8
@@ -104,6 +90,10 @@ void desktop_execute_get_result(wxString command, wxArrayString& output);
 bool create_process(const boost::filesystem::path& path, const std::wstring& cmd_opt, std::string& error_msg);
 #endif //_WIN32
 
+bool has_illegal_characters(const wxString& name);
+bool has_illegal_characters(const std::string& name);
+void show_illegal_characters_warning(wxWindow* parent);
+
 } // namespace GUI
 } // namespace Slic3r
 
