@@ -127,7 +127,7 @@ bool ObjectSettings::update_settings_list()
             optgroup->label_width = 15;
             optgroup->sidetext_width = 5;
 
-            optgroup->m_on_change = [this, config](const t_config_option_key& opt_id, const boost::any& value) {
+            optgroup->on_change = [this, config](const t_config_option_key& opt_id, const boost::any& value) {
                                     this->update_config_values(config);
                                     wxGetApp().obj_list()->changed_object(); };
 
