@@ -32,19 +32,19 @@ bool GLGizmoSeam::on_init()
 {
     m_shortcut_key = WXK_CONTROL_P;
 
-    m_desc["clipping_of_view"] = _L("Clipping of view") + ": ";
-    m_desc["reset_direction"]  = _L("Reset direction");
-    m_desc["cursor_size"]      = _L("Brush size") + ": ";
-    m_desc["cursor_type"]      = _L("Brush shape") + ": ";
-    m_desc["enforce_caption"]  = _L("Left mouse button") + ": ";
-    m_desc["enforce"]          = _L("Enforce seam");
-    m_desc["block_caption"]    = _L("Right mouse button") + ": ";
-    m_desc["block"]            = _L("Block seam");
-    m_desc["remove_caption"]   = _L("Shift + Left mouse button") + ": ";
-    m_desc["remove"]           = _L("Remove selection");
-    m_desc["remove_all"]       = _L("Remove all selection");
-    m_desc["circle"]           = _L("Circle");
-    m_desc["sphere"]           = _L("Sphere");
+    m_desc["clipping_of_view"] = _u8L("Clipping of view") + ": ";
+    m_desc["reset_direction"]  = _u8L("Reset direction");
+    m_desc["cursor_size"]      = _u8L("Brush size") + ": ";
+    m_desc["cursor_type"]      = _u8L("Brush shape") + ": ";
+    m_desc["enforce_caption"]  = _u8L("Left mouse button") + ": ";
+    m_desc["enforce"]          = _u8L("Enforce seam");
+    m_desc["block_caption"]    = _u8L("Right mouse button") + ": ";
+    m_desc["block"]            = _u8L("Block seam");
+    m_desc["remove_caption"]   = _u8L("Shift + Left mouse button") + ": ";
+    m_desc["remove"]           = _u8L("Remove selection");
+    m_desc["remove_all"]       = _u8L("Remove all selection");
+    m_desc["circle"]           = _u8L("Circle");
+    m_desc["sphere"]           = _u8L("Sphere");
 
     return true;
 }
@@ -81,42 +81,42 @@ void GLGizmoSeam::on_render_input_window(float x, float y, float bottom_limit)
 
     const float approx_height = m_imgui->scaled(12.5f);
     y = std::min(y, bottom_limit - approx_height);
-    m_imgui->set_next_window_pos(x, y, ImGuiCond_Always);
-    m_imgui->begin(get_name(), ImGuiWindowFlags_NoMove | ImGuiWindowFlags_AlwaysAutoResize | ImGuiWindowFlags_NoCollapse);
+    ImGuiPureWrap::set_next_window_pos(x, y, ImGuiCond_Always);
+    ImGuiPureWrap::begin(get_name(), ImGuiWindowFlags_NoMove | ImGuiWindowFlags_AlwaysAutoResize | ImGuiWindowFlags_NoCollapse);
 
     // First calculate width of all the texts that are could possibly be shown. We will decide set the dialog width based on that:
-    const float clipping_slider_left = std::max(m_imgui->calc_text_size(m_desc.at("clipping_of_view")).x,
-                                                m_imgui->calc_text_size(m_desc.at("reset_direction")).x)
+    const float clipping_slider_left = std::max(ImGuiPureWrap::calc_text_size(m_desc.at("clipping_of_view")).x,
+                                                ImGuiPureWrap::calc_text_size(m_desc.at("reset_direction")).x)
                                            + m_imgui->scaled(1.5f);
-    const float cursor_size_slider_left = m_imgui->calc_text_size(m_desc.at("cursor_size")).x + m_imgui->scaled(1.f);
+    const float cursor_size_slider_left = ImGuiPureWrap::calc_text_size(m_desc.at("cursor_size")).x + m_imgui->scaled(1.f);
 
-    const float cursor_type_radio_left   = m_imgui->calc_text_size(m_desc["cursor_type"]).x + m_imgui->scaled(1.f);
-    const float cursor_type_radio_sphere = m_imgui->calc_text_size(m_desc["sphere"]).x + m_imgui->scaled(2.5f);
-    const float cursor_type_radio_circle = m_imgui->calc_text_size(m_desc["circle"]).x + m_imgui->scaled(2.5f);
+    const float cursor_type_radio_left   = ImGuiPureWrap::calc_text_size(m_desc["cursor_type"]).x + m_imgui->scaled(1.f);
+    const float cursor_type_radio_sphere = ImGuiPureWrap::calc_text_size(m_desc["sphere"]).x + m_imgui->scaled(2.5f);
+    const float cursor_type_radio_circle = ImGuiPureWrap::calc_text_size(m_desc["circle"]).x + m_imgui->scaled(2.5f);
 
-    const float button_width = m_imgui->calc_text_size(m_desc.at("remove_all")).x + m_imgui->scaled(1.f);
+    const float button_width = ImGuiPureWrap::calc_text_size(m_desc.at("remove_all")).x + m_imgui->scaled(1.f);
     const float minimal_slider_width = m_imgui->scaled(4.f);
 
     float caption_max    = 0.f;
     float total_text_max = 0.f;
     for (const auto &t : std::array<std::string, 3>{"enforce", "block", "remove"}) {
-        caption_max    = std::max(caption_max, m_imgui->calc_text_size(m_desc[t + "_caption"]).x);
-        total_text_max = std::max(total_text_max, m_imgui->calc_text_size(m_desc[t]).x);
+        caption_max    = std::max(caption_max, ImGuiPureWrap::calc_text_size(m_desc[t + "_caption"]).x);
+        total_text_max = std::max(total_text_max, ImGuiPureWrap::calc_text_size(m_desc[t]).x);
     }
     total_text_max += caption_max + m_imgui->scaled(1.f);
     caption_max    += m_imgui->scaled(1.f);
 
     const float sliders_left_width = std::max(cursor_size_slider_left, clipping_slider_left);
-    const float slider_icon_width  = m_imgui->get_slider_icon_size().x;
+    const float slider_icon_width  = ImGuiPureWrap::get_slider_icon_size().x;
     float       window_width       = minimal_slider_width + sliders_left_width + slider_icon_width;
     window_width = std::max(window_width, total_text_max);
     window_width = std::max(window_width, button_width);
     window_width = std::max(window_width, cursor_type_radio_left + cursor_type_radio_sphere + cursor_type_radio_circle);
 
-    auto draw_text_with_caption = [this, &caption_max](const wxString& caption, const wxString& text) {
-        m_imgui->text_colored(ImGuiWrapper::COL_ORANGE_LIGHT, caption);
+    auto draw_text_with_caption = [&caption_max](const std::string& caption, const std::string& text) {
+        ImGuiPureWrap::text_colored(ImGuiPureWrap::COL_ORANGE_LIGHT, caption);
         ImGui::SameLine(caption_max);
-        m_imgui->text(text);
+        ImGuiPureWrap::text(text);
     };
 
     for (const auto &t : std::array<std::string, 3>{"enforce", "block", "remove"})
@@ -127,38 +127,38 @@ void GLGizmoSeam::on_render_input_window(float x, float y, float bottom_limit)
     const float max_tooltip_width = ImGui::GetFontSize() * 20.0f;
 
     ImGui::AlignTextToFramePadding();
-    m_imgui->text(m_desc.at("cursor_size"));
+    ImGuiPureWrap::text(m_desc.at("cursor_size"));
     ImGui::SameLine(sliders_left_width);
     ImGui::PushItemWidth(window_width - sliders_left_width - slider_icon_width);
     m_imgui->slider_float("##cursor_radius", &m_cursor_radius, CursorRadiusMin, CursorRadiusMax, "%.2f", 1.0f, true, _L("Alt + Mouse wheel"));
 
     ImGui::AlignTextToFramePadding();
-    m_imgui->text(m_desc.at("cursor_type"));
+    ImGuiPureWrap::text(m_desc.at("cursor_type"));
 
     float cursor_type_offset = cursor_type_radio_left + (window_width - cursor_type_radio_left - cursor_type_radio_sphere - cursor_type_radio_circle + m_imgui->scaled(0.5f)) / 2.f;
     ImGui::SameLine(cursor_type_offset);
     ImGui::PushItemWidth(cursor_type_radio_sphere);
-    if (m_imgui->radio_button(m_desc["sphere"], m_cursor_type == TriangleSelector::CursorType::SPHERE))
+    if (ImGuiPureWrap::radio_button(m_desc["sphere"], m_cursor_type == TriangleSelector::CursorType::SPHERE))
         m_cursor_type = TriangleSelector::CursorType::SPHERE;
 
     if (ImGui::IsItemHovered())
-        m_imgui->tooltip(_L("Paints all facets inside, regardless of their orientation."), max_tooltip_width);
+        ImGuiPureWrap::tooltip(_u8L("Paints all facets inside, regardless of their orientation."), max_tooltip_width);
 
     ImGui::SameLine(cursor_type_offset + cursor_type_radio_sphere);
     ImGui::PushItemWidth(cursor_type_radio_circle);
-    if (m_imgui->radio_button(m_desc["circle"], m_cursor_type == TriangleSelector::CursorType::CIRCLE))
+    if (ImGuiPureWrap::radio_button(m_desc["circle"], m_cursor_type == TriangleSelector::CursorType::CIRCLE))
         m_cursor_type = TriangleSelector::CursorType::CIRCLE;
 
     if (ImGui::IsItemHovered())
-        m_imgui->tooltip(_L("Ignores facets facing away from the camera."), max_tooltip_width);
+        ImGuiPureWrap::tooltip(_u8L("Ignores facets facing away from the camera."), max_tooltip_width);
 
     ImGui::Separator();
     if (m_c->object_clipper()->get_position() == 0.f) {
         ImGui::AlignTextToFramePadding();
-        m_imgui->text(m_desc.at("clipping_of_view"));
+        ImGuiPureWrap::text(m_desc.at("clipping_of_view"));
     }
     else {
-        if (m_imgui->button(m_desc.at("reset_direction"))) {
+        if (ImGuiPureWrap::button(m_desc.at("reset_direction"))) {
             wxGetApp().CallAfter([this](){
                     m_c->object_clipper()->set_position_by_ratio(-1., false);
                 });
@@ -172,7 +172,7 @@ void GLGizmoSeam::on_render_input_window(float x, float y, float bottom_limit)
         m_c->object_clipper()->set_position_by_ratio(clp_dist, true);
 
     ImGui::Separator();
-    if (m_imgui->button(m_desc.at("remove_all"))) {
+    if (ImGuiPureWrap::button(m_desc.at("remove_all"))) {
         Plater::TakeSnapshot snapshot(wxGetApp().plater(), _L("Reset selection"), UndoRedo::SnapshotType::GizmoAction);
         ModelObject         *mo  = m_c->selection_info()->model_object();
         int                  idx = -1;
@@ -187,7 +187,7 @@ void GLGizmoSeam::on_render_input_window(float x, float y, float bottom_limit)
         m_parent.set_as_dirty();
     }
 
-    m_imgui->end();
+    ImGuiPureWrap::end();
 }
 
 
