@@ -43,9 +43,10 @@ void BoostThreadWorker::WorkerMessage::deliver(BoostThreadWorker &runner)
 void BoostThreadWorker::run()
 {
     bool stop = false;
+
     while (!stop) {
         m_input_queue
-            .consume_one(BlockingWait{0, &m_running}, [this, &stop](JobEntry &e) {
+            .consume_one(BlockingWait{0}, [this, &stop](JobEntry &e) {
                 if (!e.job)
                     stop = true;
                 else {
@@ -60,7 +61,6 @@ void BoostThreadWorker::run()
                     e.canceled = m_canceled.load();
                     m_output_queue.push(std::move(e)); // finalization message
                 }
-                m_running.store(false);
             });
     };
 }
@@ -81,9 +81,12 @@ std::future<void> BoostThreadWorker::call_on_main_thread(std::function<void ()>
 }
 
 BoostThreadWorker::BoostThreadWorker(std::shared_ptr<ProgressIndicator> pri,
-                                     boost::thread::attributes &attribs,
-                                     const char *               name)
-    : m_progress(std::move(pri)), m_name{name}
+                                     boost::thread::attributes         &attribs,
+                                     const char                        *name)
+    : m_progress(std::move(pri))
+    , m_input_queue{m_running}
+    , m_output_queue{m_running}
+    , m_name{name}
 {
     if (m_progress)
         m_progress->set_cancel_callback([this](){ cancel(); });
