@@ -9,6 +9,7 @@
 #include <vector>
 #include <string>
 #include <boost/algorithm/string.hpp>
+#include <boost/log/trivial.hpp>
 
 #include <wx/sizer.h>
 #include <wx/stattext.h>
@@ -16,6 +17,10 @@
 #include <wx/button.h>
 #include <wx/statbox.h>
 #include <wx/wupdlock.h>
+#if wxUSE_SECRETSTORE 
+#include <wx/secretstore.h>
+#endif
+#include <wx/clipbrd.h>
 
 #include "libslic3r/libslic3r.h"
 #include "libslic3r/PrintConfig.hpp"
@@ -153,6 +158,78 @@ void PresetForPrinter::on_sys_color_changed()
     m_delete_preset_btn->sys_color_changed();
 }
 
+namespace {
+
+bool is_secret_store_ok()
+{
+#if wxUSE_SECRETSTORE 
+    wxSecretStore store = wxSecretStore::GetDefault();
+    wxString errmsg;
+    if (!store.IsOk(&errmsg)) {
+        BOOST_LOG_TRIVIAL(warning) << "wxSecretStore is not supported: " << errmsg;
+        return false;
+    }
+    return true;
+#else
+    return false;
+#endif
+}
+bool save_secret(const std::string& id, const std::string& opt, const std::string& usr, const std::string& psswd)
+{
+#if wxUSE_SECRETSTORE 
+    wxSecretStore store = wxSecretStore::GetDefault();
+    wxString errmsg;
+    if (!store.IsOk(&errmsg)) {
+        std::string msg = GUI::format("%1% (%2%).", _u8L("This system doesn't support storing passwords securely"), errmsg);
+        BOOST_LOG_TRIVIAL(error) << msg;
+        show_error(nullptr, msg);
+        return false;
+    }
+    const wxString service = GUI::format_wxstr(L"%1%/PhysicalPrinter/%2%/%3%", SLIC3R_APP_NAME, id, opt);
+    const wxString username = boost::nowide::widen(usr);
+    const wxSecretValue password(boost::nowide::widen(psswd));
+    if (!store.Save(service, username, password)) {
+        std::string msg(_u8L("Failed to save credentials to the system password store."));
+        BOOST_LOG_TRIVIAL(error) << msg;
+        show_error(nullptr, msg);
+        return false;
+    }
+    return true;
+#else
+    BOOST_LOG_TRIVIAL(error) << "wxUSE_SECRETSTORE not supported. Cannot save password to the system store.";
+    return false;
+#endif // wxUSE_SECRETSTORE 
+}
+bool load_secret(const std::string& id, const std::string& opt, std::string& usr, std::string& psswd)
+{
+#if wxUSE_SECRETSTORE
+    wxSecretStore store = wxSecretStore::GetDefault();
+    wxString errmsg;
+    if (!store.IsOk(&errmsg)) {
+        std::string msg = GUI::format("%1% (%2%).", _u8L("This system doesn't support storing passwords securely"), errmsg);
+        BOOST_LOG_TRIVIAL(error) << msg;
+        show_error(nullptr, msg);
+        return false;
+    }
+    const wxString service = GUI::format_wxstr(L"%1%/PhysicalPrinter/%2%/%3%", SLIC3R_APP_NAME, id, opt);
+    wxString username;
+    wxSecretValue password;
+    if (!store.Load(service, username, password)) {
+        std::string msg(_u8L("Failed to load credentials from the system password store."));
+        BOOST_LOG_TRIVIAL(error) << msg;
+        show_error(nullptr, msg);
+        return false;
+    }
+    usr = into_u8(username);
+    psswd = into_u8(password.GetAsString());
+    return true;
+#else
+    BOOST_LOG_TRIVIAL(error) << "wxUSE_SECRETSTORE not supported. Cannot load password from the system store.";
+    return false;
+#endif // wxUSE_SECRETSTORE 
+}
+}
+
 
 //------------------------------------------
 //          PhysicalPrinterDialog
@@ -202,6 +279,20 @@ PhysicalPrinterDialog::PhysicalPrinterDialog(wxWindow* parent, wxString printer_
         const std::set<std::string>& preset_names = printer->get_preset_names();
         for (const std::string& preset_name : preset_names)
             m_presets.emplace_back(new PresetForPrinter(this, preset_name));
+        // "stored" indicates data are stored secretly, load them from store.
+        if (m_printer.config.opt_string("printhost_password") == "stored" && m_printer.config.opt_string("printhost_password") == "stored") {
+            std::string username;
+            std::string password;
+            if (load_secret(m_printer.name, "printhost_password", username, password)) {
+                if (!username.empty())
+                    m_printer.config.opt_string("printhost_user") = username;
+                if (!password.empty())
+                    m_printer.config.opt_string("printhost_password") = password;
+            } else {
+                m_printer.config.opt_string("printhost_user") = std::string();
+                m_printer.config.opt_string("printhost_password") = std::string();
+            }
+        }
     }
 
     if (m_presets.size() == 1)
@@ -281,7 +372,7 @@ void PhysicalPrinterDialog::update_printers()
 
 void PhysicalPrinterDialog::build_printhost_settings(ConfigOptionsGroup* m_optgroup)
 {
-    m_optgroup->m_on_change = [this](t_config_option_key opt_key, boost::any value) {
+    m_optgroup->on_change = [this](t_config_option_key opt_key, boost::any value) {
         if (opt_key == "host_type" || opt_key == "printhost_authorization_type")
             this->update();
         if (opt_key == "print_host")
@@ -348,6 +439,20 @@ void PhysicalPrinterDialog::build_printhost_settings(ConfigOptionsGroup* m_optgr
         return sizer;
     };
 
+    auto api_key_copy = [=](wxWindow* parent) {
+        auto sizer = create_sizer_with_btn(parent, &m_api_key_copy_btn, "copy", _L("Copy"));
+        m_api_key_copy_btn->Bind(wxEVT_BUTTON, [=](wxCommandEvent& e) {
+            if (Field* apikey_field = m_optgroup->get_field("printhost_apikey"); apikey_field) {
+                if (wxTextCtrl* temp = dynamic_cast<wxTextCtrl*>(apikey_field->getWindow()); temp ) {
+                    wxTheClipboard->Open();
+                    wxTheClipboard->SetData(new wxTextDataObject(temp->GetValue()));
+                    wxTheClipboard->Close();
+                }
+            }
+        });
+        return sizer;
+    };
+
     // Set a wider width for a better alignment
     Option option = m_optgroup->get_option("print_host");
     option.opt.width = Field::def_width_wider();
@@ -360,7 +465,9 @@ void PhysicalPrinterDialog::build_printhost_settings(ConfigOptionsGroup* m_optgr
 
     option = m_optgroup->get_option("printhost_apikey");
     option.opt.width = Field::def_width_wider();
-    m_optgroup->append_single_option_line(option);
+    Line apikey_line = m_optgroup->create_single_option_line(option);
+    apikey_line.append_widget(api_key_copy);
+    m_optgroup->append_line(apikey_line);
 
     option = m_optgroup->get_option("printhost_port");
     option.opt.width = Field::def_width_wider();
@@ -422,6 +529,38 @@ void PhysicalPrinterDialog::build_printhost_settings(ConfigOptionsGroup* m_optgr
         m_optgroup->append_line(line);
     }
 
+    // Text line with info how passwords and api keys are stored 
+    if (is_secret_store_ok()) {
+        Line line{ "", "" };
+        line.full_width = 1;
+        line.widget = [ca_file_hint](wxWindow* parent) {
+            wxString info = GUI::format_wxstr(L"%1%:\n\t%2%\n"
+                , _L("Storing passwords")
+                , GUI::format_wxstr(_L("On this system, %1% uses the system password store to safely store and read passwords and API keys."), SLIC3R_APP_NAME));
+            auto txt = new wxStaticText(parent, wxID_ANY, info);
+            txt->SetFont(wxGetApp().normal_font());
+            auto sizer = new wxBoxSizer(wxHORIZONTAL);
+            sizer->Add(txt, 1, wxEXPAND);
+            return sizer;
+        };
+        m_optgroup->append_line(line);
+    } else {
+        Line line{ "", "" };
+        line.full_width = 1;
+        line.widget = [ca_file_hint](wxWindow* parent) {
+            wxString info = GUI::format_wxstr(L"%1%:\n\t%2%\n\t%3%\n"
+                , _L("Storing passwords")
+                , GUI::format_wxstr(_L("On this system, %1% cannot access the system password store."), SLIC3R_APP_NAME)
+                , _L("Passwords and API keys are stored in plain text."));
+            auto txt = new wxStaticText(parent, wxID_ANY, info);
+            txt->SetFont(wxGetApp().normal_font());
+            auto sizer = new wxBoxSizer(wxHORIZONTAL);
+            sizer->Add(txt, 1, wxEXPAND);
+            return sizer;
+        };
+        m_optgroup->append_line(line);
+    }
+
     for (const std::string& opt_key : std::vector<std::string>{ "printhost_user", "printhost_password" }) {        
         option = m_optgroup->get_option(opt_key);
         option.opt.width = Field::def_width_wider();
@@ -750,7 +889,7 @@ void PhysicalPrinterDialog::OnOK(wxEvent& event)
     const auto opt = m_config->option<ConfigOptionEnum<PrintHostType>>("host_type");
     if (opt->value == htPrusaConnect) {
         if (printhost_win && printhost_win->GetValue() != L"https://connect.prusa3d.com"){
-            InfoDialog msg(this, _L("Warning"), _L("URL of PrusaConnect is different from https://connect.prusa3d.com. Do you want to continue?"), true, wxYES_NO);
+            InfoDialog msg(this, _L("Warning"), _L("URL of Prusa Connect is different from https://connect.prusa3d.com. Do you want to continue?"), true, wxYES_NO);
             if(msg.ShowModal() != wxID_YES){
                 printhost_win->SetValue(L"https://connect.prusa3d.com");
                 return;
@@ -810,6 +949,13 @@ void PhysicalPrinterDialog::OnOK(wxEvent& event)
     //update printer name, if it was changed
     m_printer.set_name(into_u8(printer_name));
 
+    // save access data secretly
+    if (!m_printer.config.opt_string("printhost_password").empty()) {
+        if (save_secret(m_printer.name, "printhost_password", m_printer.config.opt_string("printhost_user"), m_printer.config.opt_string("printhost_password"))) {
+            m_printer.config.opt_string("printhost_password", false) = "stored";
+        }
+    }
+
     // save new physical printer
     printers.save_printer(m_printer, renamed_from);
 
