@@ -9,6 +9,7 @@
 #include <cmath>
 
 #include <wx/sizer.h>
+#include <wx/accel.h>
 
 #include <boost/algorithm/string/replace.hpp>
 
@@ -51,6 +52,14 @@ void sys_color_changed_menu(wxMenu* menu)
 }
 #endif /* no __linux__ */
 
+#ifndef __APPLE__
+std::vector<wxAcceleratorEntry*>& accelerator_entries_cache()
+{
+    static std::vector<wxAcceleratorEntry*> entries;
+    return entries;
+}
+#endif
+
 void enable_menu_item(wxUpdateUIEvent& evt, std::function<bool()> const cb_condition, wxMenuItem* item, wxWindow* win)
 {
     const bool enable = cb_condition();
@@ -78,8 +87,39 @@ wxMenuItem* append_menu_item(wxMenu* menu, int id, const wxString& string, const
         event_handler->Bind(wxEVT_MENU, cb, id);
     else
 #endif // __WXMSW__
+#ifndef __APPLE__
+    if (parent)
+        parent->Bind(wxEVT_MENU, cb, id);
+    else
+#endif // n__APPLE__
         menu->Bind(wxEVT_MENU, cb, id);
 
+#ifndef __APPLE__
+    if (wxAcceleratorEntry* entry = wxAcceleratorEntry::Create(string)) {
+
+        static const std::set<int> special_keys = {
+                                                WXK_PAGEUP,
+                                                WXK_PAGEDOWN,
+                                                WXK_NUMPAD_PAGEDOWN,
+                                                WXK_END,
+                                                WXK_HOME,
+                                                WXK_LEFT,
+                                                WXK_UP,
+                                                WXK_RIGHT,
+                                                WXK_DOWN,
+                                                WXK_INSERT,
+                                                WXK_DELETE,
+        };
+
+        // Check for special keys not included in the table
+        // see wxMSWKeyboard::WXWORD WXToVK(int wxk, bool *isExtended)
+        if (std::find(special_keys.begin(), special_keys.end(), entry->GetKeyCode()) == special_keys.end()) {
+            entry->SetMenuItem(item);
+            accelerator_entries_cache().push_back(entry);
+        }
+    }
+#endif
+
     if (parent) {
         parent->Bind(wxEVT_UPDATE_UI, [cb_condition, item, parent](wxUpdateUIEvent& evt) {
             enable_menu_item(evt, cb_condition, item, parent); }, id);
@@ -175,198 +215,14 @@ wxMenuItem* append_menu_check_item(wxMenu* menu, int id, const wxString& string,
     return item;
 }
 
-const unsigned int wxCheckListBoxComboPopup::DefaultWidth = 200;
-const unsigned int wxCheckListBoxComboPopup::DefaultHeight = 200;
-
-bool wxCheckListBoxComboPopup::Create(wxWindow* parent)
+void set_menu_item_bitmap(wxMenuItem* item, const std::string& icon_name)
 {
-    return wxCheckListBox::Create(parent, wxID_HIGHEST + 1, wxPoint(0, 0));
-}
-
-wxWindow* wxCheckListBoxComboPopup::GetControl()
-{
-    return this;
-}
-
-void wxCheckListBoxComboPopup::SetStringValue(const wxString& value)
-{
-    m_text = value;
-}
-
-wxString wxCheckListBoxComboPopup::GetStringValue() const
-{
-    return m_text;
-}
-
-wxSize wxCheckListBoxComboPopup::GetAdjustedSize(int minWidth, int prefHeight, int maxHeight)
-{
-    // set width dinamically in dependence of items text
-    // and set height dinamically in dependence of items count
-
-    wxComboCtrl* cmb = GetComboCtrl();
-    if (cmb != nullptr) {
-        wxSize size = GetComboCtrl()->GetSize();
-
-        unsigned int count = GetCount();
-        if (count > 0) {
-            int max_width = size.x;
-            for (unsigned int i = 0; i < count; ++i) {
-                max_width = std::max(max_width, 60 + GetTextExtent(GetString(i)).x);
-            }
-            size.SetWidth(max_width);
-            size.SetHeight(count * cmb->GetCharHeight());
-        }
-        else
-            size.SetHeight(DefaultHeight);
-
-        return size;
-    }
-    else
-        return wxSize(DefaultWidth, DefaultHeight);
-}
-
-void wxCheckListBoxComboPopup::OnKeyEvent(wxKeyEvent& evt)
-{
-    // filters out all the keys which are not working properly
-    switch (evt.GetKeyCode())
-    {
-    case WXK_LEFT:
-    case WXK_UP:
-    case WXK_RIGHT:
-    case WXK_DOWN:
-    case WXK_PAGEUP:
-    case WXK_PAGEDOWN:
-    case WXK_END:
-    case WXK_HOME:
-    case WXK_NUMPAD_LEFT:
-    case WXK_NUMPAD_UP:
-    case WXK_NUMPAD_RIGHT:
-    case WXK_NUMPAD_DOWN:
-    case WXK_NUMPAD_PAGEUP:
-    case WXK_NUMPAD_PAGEDOWN:
-    case WXK_NUMPAD_END:
-    case WXK_NUMPAD_HOME:
-    {
-        break;
-    }
-    default:
-    {
-        evt.Skip();
-        break;
-    }
-    }
-}
-
-void wxCheckListBoxComboPopup::OnCheckListBox(wxCommandEvent& evt)
-{
-    // forwards the checklistbox event to the owner wxComboCtrl
-
-    if (m_check_box_events_status == OnCheckListBoxFunction::FreeToProceed )
-    {
-        wxComboCtrl* cmb = GetComboCtrl();
-        if (cmb != nullptr) {
-            wxCommandEvent event(wxEVT_CHECKLISTBOX, cmb->GetId());
-            event.SetEventObject(cmb);
-            cmb->ProcessWindowEvent(event);
-        }
-    }
-
-    evt.Skip();
-
-    #ifndef _WIN32  // events are sent differently on OSX+Linux vs Win (more description in header file)
-        if ( m_check_box_events_status == OnCheckListBoxFunction::RefuseToProceed )
-            // this happens if the event was resent by OnListBoxSelection - next call to OnListBoxSelection is due to user clicking the text, so the function should
-            // explicitly change the state on the checkbox
-            m_check_box_events_status = OnCheckListBoxFunction::WasRefusedLastTime;
-        else
-            // if the user clicked the checkbox square, this event was sent before OnListBoxSelection was called, so we don't want it to resend it
-            m_check_box_events_status = OnCheckListBoxFunction::RefuseToProceed;
-    #endif
-}
-
-void wxCheckListBoxComboPopup::OnListBoxSelection(wxCommandEvent& evt)
-{
-    // transforms list box item selection event into checklistbox item toggle event 
-
-    int selId = GetSelection();
-    if (selId != wxNOT_FOUND)
-    {
-        #ifndef _WIN32
-            if (m_check_box_events_status == OnCheckListBoxFunction::RefuseToProceed)
-        #endif
-                Check((unsigned int)selId, !IsChecked((unsigned int)selId));
-
-        m_check_box_events_status = OnCheckListBoxFunction::FreeToProceed; // so the checkbox reacts to square-click the next time
-
-        SetSelection(wxNOT_FOUND);
-        wxCommandEvent event(wxEVT_CHECKLISTBOX, GetId());
-        event.SetInt(selId);
-        event.SetEventObject(this);
-        ProcessEvent(event);
-    }
-}
-
-
-// ***  wxDataViewTreeCtrlComboPopup  ***
-
-const unsigned int wxDataViewTreeCtrlComboPopup::DefaultWidth = 270;
-const unsigned int wxDataViewTreeCtrlComboPopup::DefaultHeight = 200;
-const unsigned int wxDataViewTreeCtrlComboPopup::DefaultItemHeight = 22;
-
-bool wxDataViewTreeCtrlComboPopup::Create(wxWindow* parent)
-{
-	return wxDataViewTreeCtrl::Create(parent, wxID_ANY/*HIGHEST + 1*/, wxPoint(0, 0), wxDefaultSize/*wxSize(270, -1)*/, wxDV_NO_HEADER);
-}
-/*
-wxSize wxDataViewTreeCtrlComboPopup::GetAdjustedSize(int minWidth, int prefHeight, int maxHeight)
-{
-	// matches owner wxComboCtrl's width
-	// and sets height dinamically in dependence of contained items count
-	wxComboCtrl* cmb = GetComboCtrl();
-	if (cmb != nullptr)
-	{
-		wxSize size = GetComboCtrl()->GetSize();
-		if (m_cnt_open_items > 0)
-			size.SetHeight(m_cnt_open_items * DefaultItemHeight);
-		else
-			size.SetHeight(DefaultHeight);
-
-		return size;
-	}
-	else
-		return wxSize(DefaultWidth, DefaultHeight);
-}
-*/
-void wxDataViewTreeCtrlComboPopup::OnKeyEvent(wxKeyEvent& evt)
-{
-	// filters out all the keys which are not working properly
-	if (evt.GetKeyCode() == WXK_UP)
-	{
-		return;
-	}
-	else if (evt.GetKeyCode() == WXK_DOWN)
-	{
-		return;
-	}
-	else
-	{
-		evt.Skip();
-		return;
-	}
-}
-
-void wxDataViewTreeCtrlComboPopup::OnDataViewTreeCtrlSelection(wxCommandEvent& evt)
-{
-	wxComboCtrl* cmb = GetComboCtrl();
-	auto selected = GetItemText(GetSelection());
-	cmb->SetText(selected);
-}
-
-// edit tooltip : change Slic3r to SLIC3R_APP_KEY
-// Temporary workaround for localization
-void edit_tooltip(wxString& tooltip)
-{
-    tooltip.Replace("Slic3r", SLIC3R_APP_KEY, true);
+    item->SetBitmap(*get_bmp_bundle(icon_name));
+#ifndef __linux__
+    const auto it = msw_menuitem_bitmaps.find(item->GetId());
+    if (it != msw_menuitem_bitmaps.end() && it->second != icon_name)
+        it->second = icon_name;
+#endif // !__linux__
 }
 
 /* Function for rescale of buttons in Dialog under MSW if dpi is changed.
@@ -473,7 +329,7 @@ std::vector<wxBitmapBundle*> get_extruder_color_icons(bool thin_icon/* = false*/
 {
     // Create the bitmap with color bars.
     std::vector<wxBitmapBundle*> bmps;
-    std::vector<std::string> colors = Slic3r::GUI::wxGetApp().plater()->get_extruder_colors_from_plater_config();
+    std::vector<std::string> colors = Slic3r::GUI::wxGetApp().plater()->get_extruder_color_strings_from_plater_config();
 
     if (colors.empty())
         return bmps;
@@ -596,196 +452,6 @@ void LockButton::update_button_bitmaps()
     Update();
 }
 
-
-
-// ----------------------------------------------------------------------------
-// ModeButton
-// ----------------------------------------------------------------------------
-
-ModeButton::ModeButton( wxWindow *          parent,
-                        wxWindowID          id,
-                        const std::string&  icon_name   /* = ""*/,
-                        const wxString&     mode        /* = wxEmptyString*/,
-                        const wxSize&       size        /* = wxDefaultSize*/,
-                        const wxPoint&      pos         /* = wxDefaultPosition*/) :
-    ScalableButton(parent, id, icon_name, mode, size, pos, wxBU_EXACTFIT)
-{
-    Init(mode);
-}
-
-ModeButton::ModeButton( wxWindow*           parent,
-                        const wxString&     mode/* = wxEmptyString*/,
-                        const std::string&  icon_name/* = ""*/,
-                        int                 px_cnt/* = 16*/) :
-    ScalableButton(parent, wxID_ANY, icon_name, mode, wxDefaultSize, wxDefaultPosition, wxBU_EXACTFIT, px_cnt)
-{
-    Init(mode);
-}
-
-ModeButton::ModeButton( wxWindow*           parent,
-                        int                 mode_id,/*ConfigOptionMode*/
-                        const wxString&     mode /*= wxEmptyString*/,
-                        int                 px_cnt /*= = 16*/) :
-    ScalableButton(parent, wxID_ANY, "", mode, wxDefaultSize, wxDefaultPosition, wxBU_EXACTFIT, px_cnt),
-    m_mode_id(mode_id)
-{
-    update_bitmap();
-    Init(mode);
-}
-
-void ModeButton::Init(const wxString &mode)
-{
-    m_tt_focused  = Slic3r::GUI::format_wxstr(_L("Switch to the %s mode"), mode);
-    m_tt_selected = Slic3r::GUI::format_wxstr(_L("Current mode is %s"),    mode);
-
-    SetBitmapMargins(3, 0);
-
-    //button events
-    Bind(wxEVT_BUTTON,          &ModeButton::OnButton, this);
-    Bind(wxEVT_ENTER_WINDOW,    &ModeButton::OnEnterBtn, this);
-    Bind(wxEVT_LEAVE_WINDOW,    &ModeButton::OnLeaveBtn, this);
-}
-
-void ModeButton::OnButton(wxCommandEvent& event)
-{
-    m_is_selected = true;
-    focus_button(m_is_selected);
-
-    event.Skip();
-}
-
-void ModeButton::SetState(const bool state)
-{
-    m_is_selected = state;
-    focus_button(m_is_selected);
-    SetToolTip(state ? m_tt_selected : m_tt_focused);
-}
-
-void ModeButton::update_bitmap()
-{
-    m_bmp = *get_bmp_bundle("mode", m_bmp_width, m_bmp_height, Slic3r::GUI::wxGetApp().get_mode_btn_color(m_mode_id));
-
-    SetBitmap(m_bmp);
-    SetBitmapCurrent(m_bmp);
-    SetBitmapPressed(m_bmp);
-}
-
-void ModeButton::focus_button(const bool focus)
-{
-    const wxFont& new_font = focus ? 
-                             Slic3r::GUI::wxGetApp().bold_font() : 
-                             Slic3r::GUI::wxGetApp().normal_font();
-
-    SetFont(new_font);
-#ifdef _WIN32
-    GetParent()->Refresh(); // force redraw a background of the selected mode button
-#else
-    SetForegroundColour(wxSystemSettings::GetColour(focus ? wxSYS_COLOUR_BTNTEXT : 
-#if defined (__linux__) && defined (__WXGTK3__)
-        wxSYS_COLOUR_GRAYTEXT
-#elif defined (__linux__) && defined (__WXGTK2__)
-        wxSYS_COLOUR_BTNTEXT
-#else 
-        wxSYS_COLOUR_BTNSHADOW
-#endif    
-    ));
-#endif /* no _WIN32 */
-
-    Refresh();
-    Update();
-}
-
-void ModeButton::sys_color_changed()
-{
-    Slic3r::GUI::wxGetApp().UpdateDarkUI(this, m_has_border);
-    update_bitmap();
-}
-
-
-// ----------------------------------------------------------------------------
-// ModeSizer
-// ----------------------------------------------------------------------------
-
-ModeSizer::ModeSizer(wxWindow *parent, int hgap/* = 0*/) :
-    wxFlexGridSizer(3, 0, hgap),
-    m_hgap_unscaled((double)(hgap)/em_unit(parent))
-{
-    SetFlexibleDirection(wxHORIZONTAL);
-
-    auto modebtnfn = [this](wxCommandEvent &event, int mode_id) {
-        if (Slic3r::GUI::wxGetApp().save_mode(mode_id))
-            event.Skip();
-        else
-            SetMode(Slic3r::GUI::wxGetApp().get_mode());
-    };
-    
-    m_mode_btns.reserve(3);
-    int mode_id = 0;
-    for (const wxString& label : {_L("Simple"), _CTX(L_CONTEXT("Advanced", "Mode"), "Mode"),_L("Expert")}) {
-        m_mode_btns.push_back(new ModeButton(parent, mode_id++, label, mode_icon_px_size()));
-
-        m_mode_btns.back()->Bind(wxEVT_BUTTON, std::bind(modebtnfn, std::placeholders::_1, int(m_mode_btns.size() - 1)));
-        Add(m_mode_btns.back());
-    }
-}
-
-void ModeSizer::SetMode(const int mode)
-{
-    for (size_t m = 0; m < m_mode_btns.size(); m++)
-        m_mode_btns[m]->SetState(int(m) == mode);
-}
-
-void ModeSizer::set_items_flag(int flag)
-{
-    for (wxSizerItem* item : this->GetChildren())
-        item->SetFlag(flag);
-}
-
-void ModeSizer::set_items_border(int border)
-{
-    for (wxSizerItem* item : this->GetChildren())
-        item->SetBorder(border);
-}
-
-void ModeSizer::sys_color_changed()
-{
-    for (ModeButton* btn : m_mode_btns)
-        btn->sys_color_changed();
-}
-
-void ModeSizer::update_mode_markers()
-{
-    for (ModeButton* btn : m_mode_btns)
-        btn->update_bitmap();
-}
-
-// ----------------------------------------------------------------------------
-// MenuWithSeparators
-// ----------------------------------------------------------------------------
-
-void MenuWithSeparators::DestroySeparators()
-{
-    if (m_separator_frst) {
-        Destroy(m_separator_frst);
-        m_separator_frst = nullptr;
-    }
-
-    if (m_separator_scnd) {
-        Destroy(m_separator_scnd);
-        m_separator_scnd = nullptr;
-    }
-}
-
-void MenuWithSeparators::SetFirstSeparator()
-{
-    m_separator_frst = this->AppendSeparator();
-}
-
-void MenuWithSeparators::SetSecondSeparator()
-{
-    m_separator_scnd = this->AppendSeparator();
-}
-
 // ----------------------------------------------------------------------------
 // PrusaBitmap
 // ----------------------------------------------------------------------------
@@ -809,6 +475,60 @@ ScalableBitmap(parent, icon_name, icon_size.x, icon_size.y, grayscale)
 {
 }
 
+ScalableBitmap::ScalableBitmap(wxWindow* parent, boost::filesystem::path& icon_path, const wxSize icon_size)
+    :m_parent(parent), m_bmp_width(icon_size.x), m_bmp_height(icon_size.y)
+{
+    wxString path = Slic3r::GUI::from_u8(icon_path.string());
+    wxBitmap bitmap;
+    const std::string ext = icon_path.extension().string();
+
+    if (ext == ".png" || ext == ".jpg") {
+        bitmap.LoadFile(path, ext == ".png" ? wxBITMAP_TYPE_PNG : wxBITMAP_TYPE_JPEG);
+
+        // check if the bitmap has a square shape
+
+        if (wxSize sz = bitmap.GetSize(); sz.x != sz.y) {
+            const int bmp_side = std::min(sz.GetWidth(), sz.GetHeight());
+
+            wxRect rc = sz.GetWidth() > sz.GetHeight() ?
+                        wxRect(int( 0.5 * (sz.x - sz.y)), 0, bmp_side, bmp_side) :
+                        wxRect(0, int( 0.5 * (sz.y - sz.x)), bmp_side, bmp_side) ;
+
+            bitmap = bitmap.GetSubBitmap(rc);
+        }
+
+        // set mask for circle shape
+
+        wxBitmapBundle mask_bmps = *get_bmp_bundle("user_mask", bitmap.GetSize().GetWidth());
+        wxMask* mask = new wxMask(mask_bmps.GetBitmap(bitmap.GetSize()), *wxBLACK);
+        bitmap.SetMask(mask);
+
+        // get allowed scale factors
+
+        std::set<double> scales = { 1.0 };
+#ifdef __APPLE__
+        scales.emplace(Slic3r::GUI::mac_max_scaling_factor());
+#elif _WIN32
+        size_t disp_cnt = wxDisplay::GetCount();
+        for (size_t disp = 0; disp < disp_cnt; ++disp)
+            scales.emplace(wxDisplay(disp).GetScaleFactor());
+#endif
+
+        // create bitmaps for bundle
+
+        wxVector<wxBitmap> bmps;
+        for (double scale : scales) {
+            wxBitmap bmp = bitmap;
+            wxBitmap::Rescale(bmp, icon_size * scale);
+            bmps.push_back(bmp);
+        }
+        m_bmp = wxBitmapBundle::FromBitmaps(bmps);
+    }
+    else if (ext == ".svg") {
+        m_bmp = wxBitmapBundle::FromSVGFile(path, icon_size);
+    }
+}
+
 void ScalableBitmap::sys_color_changed()
 {
     m_bmp = *get_bmp_bundle(m_icon_name, m_bmp_width, m_bmp_height);
@@ -868,10 +588,15 @@ ScalableButton::ScalableButton( wxWindow *          parent,
     SetBitmap(bitmap.bmp());
 }
 
-void ScalableButton::SetBitmap_(const ScalableBitmap& bmp)
+void ScalableButton::SetBitmap_(const ScalableBitmap& bitmap)
 {
-    SetBitmap(bmp.bmp());
-    m_current_icon_name = bmp.name();
+    const wxBitmapBundle& bmp = bitmap.bmp();
+    SetBitmap(bmp);
+    SetBitmapCurrent(bmp);
+    SetBitmapPressed(bmp);
+    SetBitmapFocus(bmp);
+    SetBitmapDisabled(bmp);
+    m_current_icon_name = bitmap.name();
 }
 
 bool ScalableButton::SetBitmap_(const std::string& bmp_name)
@@ -904,10 +629,20 @@ int ScalableButton::GetBitmapHeight()
 #endif
 }
 
+wxSize ScalableButton::GetBitmapSize()
+{
+#ifdef __APPLE__
+    return wxSize(GetBitmap().GetScaledWidth(), GetBitmap().GetScaledHeight());
+#else
+    return wxSize(GetBitmap().GetWidth(), GetBitmap().GetHeight());
+#endif
+}
+
 void ScalableButton::sys_color_changed()
 {
     Slic3r::GUI::wxGetApp().UpdateDarkUI(this, m_has_border);
-
+    if (m_current_icon_name.empty())
+        return;
     wxBitmapBundle bmp = *get_bmp_bundle(m_current_icon_name, m_bmp_width, m_bmp_height);
     SetBitmap(bmp);
     SetBitmapCurrent(bmp);
