@@ -557,6 +557,12 @@ Http& Http::set_post_body(const std::string &body)
 	return *this;
 }
 
+Http& Http::set_referer(const std::string& referer)
+{
+	if (p) { ::curl_easy_setopt(p->curl, CURLOPT_REFERER, referer.c_str()); }
+	return *this;
+}
+
 Http& Http::set_put_body(const fs::path &path)
 {
 	if (p) { p->set_put_body(path);}
@@ -587,6 +593,22 @@ Http& Http::on_ip_resolve(IPResolveFn fn)
 	return *this;
 }
 
+Http& Http::cookie_file(const std::string& file_path)
+{
+	if (p) {
+		::curl_easy_setopt(p->curl, CURLOPT_COOKIEFILE, file_path.c_str());
+	}
+	return *this;
+}
+
+Http& Http::cookie_jar(const std::string& file_path)
+{
+	if (p) {
+		::curl_easy_setopt(p->curl, CURLOPT_COOKIEJAR, file_path.c_str());
+	}
+	return *this;
+}
+
 Http::Ptr Http::perform()
 {
 	auto self = std::make_shared<Http>(std::move(*this));
